/*	dooroos : drv_keybd.c, v 0.1 2012-12-12 */

/*
 * Copyright (C) 2012	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: drv_keybd.c
 *
 * Purpose: 
 */

#include <types.h>
#include <thread.h>
#include <message.h>
#include <interrupt.h>
#include <window.h>
#include <time.h>
#include <device.h>
#include <error.h>

#include <mach/hal_interrupt.h>

#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>



#include <hal_panic.h>

#include "key.h"

#define KBD_PB3_IRQ_PIN_NUM     27
#define KEY_PAD_PB3             (1<<KBD_PB3_IRQ_PIN_NUM)

STATIC UINT32      gnSysIntr = 0;

//==========================================================================
STATIC BOOL                     th_KeybdDevice( UINT32 Arg );

#define KEYBDDEV_STACK_SIZE     (512)
STATIC UINT32                   KeybdDev_Stack[KEYBDDEV_STACK_SIZE>>2];

STATIC DEVICECONFIG             Keybd_DeviceConfig ={"KBD", 0, -1, NULL, NULL };

PUBLIC THREADCONFIG             KeybdDev_Thread =
                                            {
                                                (UINT32)&Keybd_DeviceConfig,
                                                th_KeybdDevice,
                                                (PBYTE)KeybdDev_Stack,
                                                KEYBDDEV_STACK_SIZE,
                                                THREAD_PRIORITY_DRIVER_HIGH+4
                                            };
//==========================================================================

#define KEYBD_TIMER_DELAY       100
#define KEYBD_TIMER_BITMAP      0x100

STATIC UINT32       PrevKeyNum = KEY_NO_INPUT;
STATIC INPUTEVENT   KeyInputEvent;

//STATIC HANDLE       hKeybdSoftTimer;
STATIC UINT32       Channel;

//==========================================================================

STATIC VOID
KeybdInit( VOID )
{
	AT91PS_PMC  pPMC = (AT91PS_PMC)AT91C_BASE_PMC;
    pPMC->PMC_PCER0 = (1 << SAMA5D3_ID_PIOE );       //power up the controller

    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP3_LEFT );
    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP3_RIGHT );
    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP3_UP );
    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP3_DOWN );
    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP3_PUSH );
    //AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_BP4 );
    AT91F_PIO_PullupActive( AT91C_BASE_PIOE, KEY_PAD_PB3 );

    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP3_LEFT );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP3_RIGHT );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP3_UP );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP3_DOWN );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP3_PUSH );
    //AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_BP4 );	
    AT91F_PIO_CfgInput( AT91C_BASE_PIOE, KEY_PAD_PB3 );	

kprintf(" KeybdInit : initialize the keyboard gpio\n");

	return;
}


STATIC UINT32
Keybd_ReadInput( VOID )
{
    UINT32  RetKey = KEY_NO_INPUT;

//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP3_LEFT) )
//	    RetKey = VK_LEFT;
//    else
//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP3_RIGHT) )
//	    RetKey = VK_RIGHT;
//    else
//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP3_UP) )
//	    RetKey = VK_UP;
//    else
//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP3_DOWN) )
//	    RetKey = VK_DOWN;
//    else
//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP3_PUSH) )
//	    RetKey = VK_RETURN;
//    else
//	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOB, KEY_PAD_BP4) )
//	    RetKey = VK_F1;
//    else
	if( !AT91F_PIO_IsInputSet(AT91C_BASE_PIOE, KEY_PAD_PB3) )
	    RetKey = VK_F1;

DEBUG2(1,"th_DrvKeybd RetKey = %x\n",RetKey);
	return RetKey;

}

/*
  Name: th_KeybdDevice()
  Copyright: 
  Author: jung byoung oh
  Date: 15-12-03 11:37
  Description: block device thread
  Command:
    MESSAGE_TYPE_INTERRUPT
    MESSAGE_TYPE_DEVICE_READ
    MESSAGE_TYPE_DEVICE_WRITE
*/
STATIC BOOL
th_KeybdDevice( UINT32 Arg )
{
	JB_RECVMESSAGE      Message;
    HANDLE              hPort;

    hPort = PortGet( );    
    PortSetType(hPort, PORTTYPE_MESSAGE|PORTTYPE_INTERRUPT|PORTTYPE_SIGNAL);    // Only Receive the request

//DEBUG2(1,"th_DrvKeybd %X\n",Arg);

    Keybd_DeviceConfig.Name[3] = (UINT32)Keybd_DeviceConfig.Minor + '1';
    Keybd_DeviceConfig.Channel = Channel = ChannelGet( hPort );
//DEBUG2(1,"th_DrvKeybd p_DeviceConfig->Channel = %X\n",Keybd_DeviceConfig.Channel);
    DeviceRegister( &Keybd_DeviceConfig );
//DEBUG2(1,"th_DrvKeybd p_DeviceConfig->Channel = %X\n",Keybd_DeviceConfig.Channel);

    //hKeybdSoftTimer = SoftTimerCreate( Channel, KEYBD_TIMER_DELAY, KEYBD_TIMER_BITMAP );

    KeybdInit();

    {
        SYSINTR_REQUEST		SysIntrRequest;
        UINT32 nSysIntr;
        SysIntrRequest.nHalIrq = IRQ_VIR_GPIO_E_START + KBD_PB3_IRQ_PIN_NUM;
        SysIntrRequest.BitMap = 1<<0;
        SysIntrRequest.Flag = SYSINTR_REQ_FLAG_STATIC;

        if( !KernelIoControl( IOCTRL_HAL_SYSINTR_REQUEST, &SysIntrRequest, sizeof(SYSINTR_REQUEST), &nSysIntr, sizeof(UINT32)))
        {
            DEBUGMSG(1,"%s (%d) : nSysIntr = %d\n", __FUNCTION__, __LINE__, nSysIntr);
            return;
        }
        DEBUGMSG(1,"%s (%d) : VBUS nSysIntr = %d\n", __FUNCTION__, __LINE__, nSysIntr);
        gnSysIntr = nSysIntr;
    }

    InterruptRegister(gnSysIntr, hPort);
//DEBUG2(1,"th_DrvKeybd hPort = %X\n",hPort);

    InterruptEnable(gnSysIntr);

    //// start the timer
    //if( hKeybdSoftTimer != NULL )
    //    SoftTimerStart(hKeybdSoftTimer); 

	while(1)
	{
        //kprintf("th_KeybdDevice MessageReceive now port = %x\n", hPort);
		MessageReceive( hPort, &Message);

        //kprintf("th_KeybdDevice MessageReceive Message.Type = %x\n", Message.Type);
		switch(Message.Type)
		{

        case MESSAGE_TYPE_INTERRUPT:
        {
            UINT32  KeyNum;
            
//kprintf("th_KeybdDevice MESSAGE_TYPE_INTERRUPT %x\n",Message.Param);
            KeyNum = Keybd_ReadInput();
#if 0            
            if( PrevKeyNum != KeyNum )
            {
                KeyInputEvent.Type = INPUTEVENT_TYPE_KEY;
                KeyInputEvent.u.Key.vScan = NULL;       // do not use here

                if( PrevKeyNum != KEY_NO_INPUT )
                {
                    KeyInputEvent.u.Key.State = KEY_STATE_RELEASED;
                    KeyInputEvent.u.Key.vKey = PrevKeyNum;
                    SendInputEvent( &KeyInputEvent );
                }
                
                if( KeyNum != KEY_NO_INPUT )
                {
                    KeyInputEvent.u.Key.State = KEY_STATE_PUSHED;
                    KeyInputEvent.u.Key.vKey = KeyNum;
                    SendInputEvent( &KeyInputEvent );
                }
kprintf("th_KeybdDevice: PrevKeyNum = %x\n",PrevKeyNum);
kprintf("th_KeybdDevice: KeyNum = %x\n",KeyNum);
            }
#endif

            PrevKeyNum = KeyNum;

            InterruptDone( gnSysIntr );
            continue;
        }
        case MESSAGE_TYPE_SIGNAL:
		{
            continue;
        }

		case MESSAGE_TYPE_DEVICE_CONTROL:
        {

            Message.Type = TRUE;


            break;
	    }

		default:
			continue;
		}

		MessageReply( &Message );
	}

	return TRUE;	
}



#include <moduleloader.h>

PUBLIC BOOL
DllMain( HANDLE hDll, UINT32 Reason, PVOID Reserved )
{
    switch( Reason )
    {
    case DLL_PROCESS_ATTACH:
        ThreadCreate (&KeybdDev_Thread);
    	break;
    
    case DLL_PROCESS_DETACH:
    	break;
    }

    return TRUE;
}






